@@ -1,9 +1,9 @@
 /*
- * "$Id: file.c,v 1.13.2.40 2002/10/03 17:43:17 mike Exp $"
+ * "$Id: file.c,v 1.13.2.51 2004/10/23 19:57:15 mike Exp $"
  *
  *   Filename routines for HTMLDOC, a HTML document processing program.
  *
- *   Copyright 1997-2002 by Easy Software Products.
+ *   Copyright 1997-2004 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -15,7 +15,7 @@
  *       Attn: ESP Licensing Information
  *       Easy Software Products
  *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636-3111 USA
+ *       Hollywood, Maryland 20636-3142 USA
  *
  *       Voice: (301) 373-9600
  *       EMail: info@easysw.com
@@ -26,6 +26,7 @@
  *   file_basename()    - Return the base filename without directory or target.
  *   file_cleanup()     - Close an open HTTP connection and remove
  *                        temporary files...
+ *   file_cookies()     - Set the HTTP cookies for remote accesses.
  *   file_directory()   - Return the directory without filename or target.
  *   file_extension()   - Return the extension of a file without the target.
  *   file_find_check()  - Check to see if the specified file or URL exists...
@@ -69,11 +70,11 @@
 
 #ifdef WIN32
 #  define getpid	GetCurrentProcessId
-#  define TEMPLATE	"%s%08x.%06d.tmp"
+#  define TEMPLATE	"%s/%08lx.%06d.tmp"
 #  define OPENMODE	(_O_CREAT | _O_RDWR | _O_TRUNC | _O_BINARY)
 #  define OPENPERM	(_S_IREAD | _S_IWRITE)
 #else
-#  define TEMPLATE	"%s/%06d.%06d.tmp"
+#  define TEMPLATE	"%s/%06ld.%06d.tmp"
 #  define OPENMODE	(O_CREAT | O_RDWR | O_EXCL | O_TRUNC)
 #  define OPENPERM	0600
 #endif /* WIN32 */
@@ -101,6 +102,7 @@ int	web_files = 0,			/* Number of temporary files */
 	web_alloc = 0;			/* Number of allocated files */
 cache_t	*web_cache = NULL;		/* Cache array */
 int	no_local = 0;			/* Non-zero to disable local files */
+char	cookies[1024] = "";		/* HTTP cookies, if any */
 
 
 /*
@@ -121,10 +123,6 @@ file_basename(const char *s)	/* I - Filename or URL */
     basename ++;
   else if ((basename = strrchr(s, '\\')) != NULL)
     basename ++;
-#ifdef MAC
-  else if ((basename = strrchr(s, ':')) != NULL)
-    basename ++;
-#endif /* MAC */
   else
     basename = (char *)s;
 
@@ -134,8 +132,7 @@ file_basename(const char *s)	/* I - Filename or URL */
   if (strchr(basename, '#') == NULL)
     return (basename);
 
-  strncpy(buf, basename, sizeof(buf) - 1);
-  buf[sizeof(buf) - 1] = '\0';
+  strlcpy(buf, basename, sizeof(buf));
   *(char *)strchr(buf, '#') = '\0';
 
   return (buf);
@@ -151,10 +148,9 @@ file_cleanup(void)
 {
   int		i;			/* Looping var */
   char		filename[1024];		/* Temporary file */
-#ifdef WIN32
-  char		tmpdir[1024];		/* Temporary directory */
-#else
   const char	*tmpdir;		/* Temporary directory */
+#ifdef WIN32
+  char		tmppath[1024];		/* Temporary directory */
 #endif /* WIN32 */
   const char	*debug;			/* HTMLDOC_DEBUG env var */
 
@@ -166,7 +162,11 @@ file_cleanup(void)
   }
 
 #ifdef WIN32
-  GetTempPath(sizeof(tmpdir), tmpdir);
+  if ((tmpdir = getenv("TEMP")) == NULL)
+  {
+    GetTempPath(sizeof(tmppath), tmppath);
+    tmpdir = tmppath;
+  }
 #else
   if ((tmpdir = getenv("TMPDIR")) == NULL)
     tmpdir = "/var/tmp";
@@ -192,7 +192,7 @@ file_cleanup(void)
     for (i = 0; i < web_files; i ++)
     {
       snprintf(filename, sizeof(filename), TEMPLATE, tmpdir,
-               getpid(), i + 1);
+               (long)getpid(), i + 1);
       progress_error(HD_ERROR_NONE, "DEBUG: %-31.31s %s\n",
                      web_cache[i].url ? web_cache[i].url : "none", filename);
     }
@@ -205,7 +205,7 @@ file_cleanup(void)
   while (web_files > 0)
   {
     snprintf(filename, sizeof(filename), TEMPLATE, tmpdir,
-             getpid(), web_files);
+             (long)getpid(), web_files);
 
     if (unlink(filename))
       progress_error(HD_ERROR_DELETE_ERROR,
@@ -230,6 +230,20 @@ file_cleanup(void)
 }
 
 
+/*
+ * 'file_cookies()' - Set the HTTP cookies for remote accesses.
+ */
+
+void
+file_cookies(const char *s)		/* I - Cookie string or NULL */
+{
+  if (s)
+    strlcpy(cookies, s, sizeof(cookies));
+  else
+    cookies[0] = '\0';
+}
+
+
 /*
  * 'file_directory()' - Return the directory without filename or target.
  */
@@ -274,24 +288,20 @@ file_directory(const char *s)	/* I - Filename or URL */
     * Normal stuff...
     */
 
-    strncpy(buf, s, sizeof(buf) - 1);
-    buf[sizeof(buf) - 1] = '\0';
+    strlcpy(buf, s, sizeof(buf));
 
     if ((dir = strrchr(buf, '/')) != NULL)
       *dir = '\0';
     else if ((dir = strrchr(buf, '\\')) != NULL)
       *dir = '\0';
-#ifdef MAC
-    else if ((dir = strrchr(buf, ':')) != NULL)
-      *dir = '\0';
-#endif /* MAC */
     else
       return (".");
 
     if (strncmp(buf, "file:", 5) == 0)
-      strcpy(buf, buf + 5);
+      hd_strcpy(buf, buf + 5);
 
     if (!buf[0])
+      /* Safe because buf is more than 2 chars long */
       strcpy(buf, "/");
   }
 
@@ -317,10 +327,6 @@ file_extension(const char *s)	/* I - Filename or URL */
     extension ++;
   else if ((extension = strrchr(s, '\\')) != NULL)
     extension ++;
-#ifdef MAC
-  else if ((extension = strrchr(s, ':')) != NULL)
-    extension ++;
-#endif /* MAC */
   else
     extension = s;
 
@@ -332,8 +338,7 @@ file_extension(const char *s)	/* I - Filename or URL */
   if (strchr(extension, '#') == NULL)
     return (extension);
 
-  strncpy(buf, extension, sizeof(buf) - 1);
-  buf[sizeof(buf) - 1] = '\0';
+  strlcpy(buf, extension, sizeof(buf));
 
   *(char *)strchr(buf, '#') = '\0';
 
@@ -371,10 +376,10 @@ file_find_check(const char *filename)	/* I - File or URL */
 
   if (strncmp(filename, "http:", 5) == 0 || strncmp(filename, "//", 2) == 0)
     strcpy(method, "http");
-#ifdef HAVE_LIBSSL
+#ifdef HAVE_SSL
   else if (strncmp(filename, "https:", 6) == 0)
     strcpy(method, "https");
-#endif /* HAVE_LIBSSL */
+#endif /* HAVE_SSL */
   else
     strcpy(method, "file");
 
@@ -437,8 +442,7 @@ file_find_check(const char *filename)	/* I - File or URL */
 
         connhost = hostname;
         connport = port;
-        strncpy(connpath, resource, sizeof(connpath) - 1);
-	connpath[sizeof(connpath) - 1] = '\0';
+        strlcpy(connpath, resource, sizeof(connpath));
       }
 
       if (http != NULL && strcasecmp(http->hostname, hostname) != 0)
@@ -450,16 +454,15 @@ file_find_check(const char *filename)	/* I - File or URL */
       if (http == NULL)
       {
         progress_show("Connecting to %s...", connhost);
-        atexit(file_cleanup);
 
-#ifdef HAVE_LIBSSL
+#ifdef HAVE_SSL
         if (strcmp(method, "http") == 0)
           http = httpConnect(connhost, connport);
 	else
           http = httpConnectEncrypt(connhost, connport, HTTP_ENCRYPT_ALWAYS);
 #else
         http = httpConnect(connhost, connport);
-#endif /* HAVE_LIBSSL */
+#endif /* HAVE_SSL */
 
         if (http == NULL)
 	{
@@ -484,6 +487,8 @@ file_find_check(const char *filename)	/* I - File or URL */
         httpSetField(http, HTTP_FIELD_AUTHORIZATION, connauth);
       }
 
+      httpSetCookie(http, cookies);
+
       if (!httpGet(http, connpath))
       {
 	while ((status = httpUpdate(http)) == HTTP_CONTINUE);
@@ -535,7 +540,7 @@ file_find_check(const char *filename)	/* I - File or URL */
     {
       count += bytes;
       progress_update((100 * count / total) % 101);
-      fwrite(resource, 1, bytes, fp);
+      fwrite(resource, 1, (size_t)bytes, fp);
     }
 
     progress_hide();
@@ -599,10 +604,7 @@ file_find(const char *path,		/* I - Path "dir;dir;dir" */
   */
 
   if (strchr(s, '%') == NULL)
-  {
-    strncpy(basename, s, sizeof(basename) - 1);
-    basename[sizeof(basename) - 1] = '\0';
-  }
+    strlcpy(basename, s, sizeof(basename));
   else
   {
     for (sptr = s, temp = basename;
@@ -681,7 +683,7 @@ file_find(const char *path,		/* I - Path "dir;dir;dir" */
           basename[0] != '/')
 	*temp++ = '/';
 
-      strncpy(temp, basename, sizeof(filename) - (temp - filename) - 1);
+      strlcpy(temp, basename, sizeof(filename) - (temp - filename));
 
      /*
       * See if the file or URL exists...
@@ -833,10 +835,7 @@ file_localize(const char *filename,	/* I - Filename */
     sprintf(temp, "%s/%s", cwd, newslash);
   }
   else
-  {
-    strncpy(temp, filename, sizeof(temp) - 1);
-    temp[sizeof(temp) - 1] = '\0';
-  }
+    strlcpy(temp, filename, sizeof(temp));
 
   for (slash = temp, newslash = newcwd;
        *slash != '\0' && *newslash != '\0';
@@ -867,11 +866,11 @@ file_localize(const char *filename,	/* I - Filename */
   while (*newslash != '\0')
   {
     if (*newslash == '/' || *newslash == '\\')
-      strcat(newfilename, "../");
+      strlcat(newfilename, "../", sizeof(newfilename));
     newslash ++;
   }
 
-  strcat(newfilename, slash);
+  strlcat(newfilename, slash, sizeof(newfilename));
 
   return (newfilename);
 }
@@ -935,8 +934,7 @@ file_proxy(const char *url)	/* I - URL of proxy server */
 
     if (strcmp(method, "http") == 0)
     {
-      strncpy(proxy_host, hostname, sizeof(proxy_host) - 1);
-      proxy_host[sizeof(proxy_host) - 1] = '\0';
+      strlcpy(proxy_host, hostname, sizeof(proxy_host));
       proxy_port = port;
     }
   }
@@ -961,10 +959,6 @@ file_target(const char *s)	/* I - Filename or URL */
     basename ++;
   else if ((basename = strrchr(s, '\\')) != NULL)
     basename ++;
-#ifdef MAC
-  else if ((basename = strrchr(s, ':')) != NULL)
-    basename ++;
-#endif /* MAC */
   else
     basename = s;
 
@@ -986,10 +980,9 @@ file_temp(char *name,			/* O - Filename */
   cache_t	*temp;			/* Pointer to cache entry */
   FILE		*fp;			/* File pointer */
   int		fd;			/* File descriptor */
-#ifdef WIN32
-  char		tmpdir[1024];		/* Buffer for temp dir */
-#else
   const char	*tmpdir;		/* Temporary directory */
+#ifdef WIN32
+  char		tmppath[1024];		/* Buffer for temp dir */
 #endif /* WIN32 */
 
 
@@ -1028,13 +1021,17 @@ file_temp(char *name,			/* O - Filename */
   web_files ++;
 
 #ifdef WIN32
-  GetTempPath(sizeof(tmpdir), tmpdir);
+  if ((tmpdir = getenv("TEMP")) == NULL)
+  {
+    GetTempPath(sizeof(tmppath), tmppath);
+    tmpdir = tmppath;
+  }
 #else
   if ((tmpdir = getenv("TMPDIR")) == NULL)
     tmpdir = "/var/tmp";
 #endif /* WIN32 */
 
-  snprintf(name, len, TEMPLATE, tmpdir, getpid(), web_files);
+  snprintf(name, (size_t)len, TEMPLATE, tmpdir, (long)getpid(), web_files);
 
   if ((fd = open(name, OPENMODE, OPENPERM)) >= 0)
     fp = fdopen(fd, "w+b");
@@ -1051,5 +1048,5 @@ file_temp(char *name,			/* O - Filename */
 
 
 /*
- * End of "$Id: file.c,v 1.13.2.40 2002/10/03 17:43:17 mike Exp $".
+ * End of "$Id: file.c,v 1.13.2.51 2004/10/23 19:57:15 mike Exp $".
  */
