@@ -1,9 +1,9 @@
 /*
- * "$Id: html.h,v 1.9.2.10 2002/07/27 03:41:31 mike Exp $"
+ * "$Id: html.h,v 1.9.2.15 2004/05/05 18:58:40 mike Exp $"
  *
  *   HTML parsing definitions for HTMLDOC, a HTML document processing program.
  *
- *   Copyright 1997-2002 by Easy Software Products.
+ *   Copyright 1997-2004 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -15,7 +15,7 @@
  *       Attn: ESP Licensing Information
  *       Easy Software Products
  *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636-3111 USA
+ *       Hollywood, Maryland 20636-3142 USA
  *
  *       Voice: (301) 373-9600
  *       EMail: info@easysw.com
@@ -45,10 +45,10 @@ extern "C" {
  * Define some compatibility macros for Microsoft Windows...
  */
 
-#  if defined(WIN32) || defined(__EMX__)
+#  ifdef WIN32
 #    define strcasecmp(s,t)	stricmp(s,t)
 #    define strncasecmp(s,t,n)	strnicmp(s,t,n)
-#  endif /* WIN32 || __EMX__ */
+#  endif /* WIN32 */
 
 
 /*
@@ -299,6 +299,10 @@ extern int	htmlDeleteTree(tree_t *parent);
 extern tree_t	*htmlInsertTree(tree_t *parent, markup_t markup, uchar *data);
 extern tree_t	*htmlNewTree(tree_t *parent, markup_t markup, uchar *data);
 
+extern tree_t	*htmlFindFile(tree_t *doc, uchar *filename);
+extern tree_t	*htmlFindTarget(tree_t *doc, uchar *name);
+extern void	htmlFixLinks(tree_t *doc, tree_t *tree, uchar *base = 0);
+
 extern uchar	*htmlGetText(tree_t *tree);
 extern uchar	*htmlGetMeta(tree_t *tree, uchar *name);
 
@@ -320,5 +324,5 @@ extern void	htmlDebugStats(const char *title, tree_t *t);
 #endif /* !_HTML_H_ */
 
 /*
- * End of "$Id: html.h,v 1.9.2.10 2002/07/27 03:41:31 mike Exp $".
+ * End of "$Id: html.h,v 1.9.2.15 2004/05/05 18:58:40 mike Exp $".
  */
