@@ -1,9 +1,9 @@
 /*
- * "$Id: htmldoc.h,v 1.18.2.18 2002/06/05 03:59:34 mike Exp $"
+ * "$Id: htmldoc.h,v 1.18.2.25 2004/06/15 01:09:09 mike Exp $"
  *
  *   Header file for HTMLDOC, a HTML document processing program.
  *
- *   Copyright 1997-2002 by Easy Software Products.
+ *   Copyright 1997-2004 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -15,7 +15,7 @@
  *       Attn: ESP Licensing Information
  *       Easy Software Products
  *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636-3111 USA
+ *       Hollywood, Maryland 20636-3142 USA
  *
  *       Voice: (301) 373-9600
  *       EMail: info@easysw.com
@@ -139,6 +139,7 @@ enum	/* PDF document permissions */
 
 VAR int		Verbosity	VALUE(0);	/* Verbosity */
 VAR int		StrictHTML	VALUE(0);	/* Do strict HTML checking */
+VAR int		CGIMode		VALUE(0);	/* Running as CGI? */
 VAR int		Errors		VALUE(0);	/* Number of errors */
 VAR int		Compression	VALUE(1);	/* Non-zero means compress PDFs */
 VAR int		TitlePage	VALUE(1),	/* Need a title page */
@@ -148,7 +149,7 @@ VAR int		TitlePage	VALUE(1),	/* Need a title page */
 		TocDocCount	VALUE(0);	/* Number of chapters */
 VAR int		OutputType	VALUE(OUTPUT_BOOK);
 						/* Output a "book", etc. */
-VAR char	OutputPath[255]	VALUE("");	/* Output directory/name */
+VAR char	OutputPath[1024] VALUE("");	/* Output directory/name */
 VAR int		OutputFiles	VALUE(0),	/* Generate multiple files? */
 		OutputColor	VALUE(1);	/* Output color images */
 VAR int		OutputJPEG	VALUE(0);	/* JPEG compress images? */
@@ -166,7 +167,7 @@ VAR int		Encryption	VALUE(0),	/* Encrypt the PDF file? */
 		Permissions	VALUE(-4);	/* File permissions? */
 VAR char	OwnerPassword[33] VALUE(""),	/* Owner password */
 		UserPassword[33] VALUE("");	/* User password */
-VAR int		EmbedFonts	VALUE(0);	/* Embed fonts? */
+VAR int		EmbedFonts	VALUE(1);	/* Embed fonts? */
 VAR int		PSLevel		VALUE(2),	/* Language level (0 for PDF) */
 		PSCommands	VALUE(0),	/* Output PostScript commands? */
 		XRXComments	VALUE(0);	/* Output Xerox comments? */
@@ -195,11 +196,18 @@ VAR char	*Header[3]	NULL3,		/* Header for regular pages */
 		TocTitle[1024]	VALUE("Table of Contents");
 						/* TOC title string */
 
-VAR char	TitleImage[255]	VALUE(""),	/* Title page image */
-		LogoImage[255]	VALUE(""),	/* Logo image */
+VAR char	TitleImage[1024] VALUE(""),	/* Title page image */
+		LogoImage[1024]	VALUE(""),	/* Logo image */
 		BodyColor[255]	VALUE(""),	/* Body color */
-		BodyImage[255]	VALUE(""),	/* Body image */
+		BodyImage[1024]	VALUE(""),	/* Body image */
 		LinkColor[255]	VALUE("");	/* Link color */
+
+VAR char	HFImage[MAX_HF_IMAGES][1024]	/* Header/footer images */
+#  ifdef _HTMLDOC_CXX_
+= { "" }
+#  endif /* _HTMLDOC_CXX_ */
+;
+
 VAR int		LinkStyle	VALUE(1);	/* 1 = underline, 0 = plain */
 VAR int		Links		VALUE(1);	/* 1 = generate links, 0 = no links */
 VAR char	Path[2048]	VALUE(""),	/* Search path */
@@ -247,6 +255,8 @@ extern int	pspdf_export(tree_t *document, tree_t *toc);
 
 extern int	html_export(tree_t *document, tree_t *toc);
 
+extern int	htmlsep_export(tree_t *document, tree_t *toc);
+
 extern tree_t	*toc_build(tree_t *tree);
 
 extern void	get_color(const uchar *c, float *rgb, int defblack = 1);
@@ -265,5 +275,5 @@ extern char	*format_number(int n, char f);
 #endif /* __cplusplus */
 
 /*
- * End of "$Id: htmldoc.h,v 1.18.2.18 2002/06/05 03:59:34 mike Exp $".
+ * End of "$Id: htmldoc.h,v 1.18.2.25 2004/06/15 01:09:09 mike Exp $".
  */
