@@ -1,9 +1,9 @@
 /*
- * "$Id: htmllib.cxx,v 1.41.2.64 2002/10/11 14:23:28 mike Exp $"
+ * "$Id: htmllib.cxx,v 1.41.2.79 2004/10/21 19:05:04 mike Exp $"
  *
  *   HTML parsing routines for HTMLDOC, a HTML document processing program.
  *
- *   Copyright 1997-2002 by Easy Software Products.
+ *   Copyright 1997-2004 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -15,7 +15,7 @@
  *       Attn: ESP Licensing Information
  *       Easy Software Products
  *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636-3111 USA
+ *       Hollywood, Maryland 20636-3142 USA
  *
  *       Voice: (301) 373-9600
  *       EMail: info@easysw.com
@@ -258,32 +258,31 @@ static uchar	indent[255] = "";
  * 'htmlReadFile()' - Read a file for HTML markup codes.
  */
 
-tree_t *			/* O - Pointer to top of file tree */
-htmlReadFile(tree_t     *parent,/* I - Parent tree entry */
-             FILE       *fp,	/* I - File pointer */
-	     const char *base)	/* I - Base directory for file */
+tree_t *				// O - Pointer to top of file tree
+htmlReadFile(tree_t     *parent,	// I - Parent tree entry
+             FILE       *fp,		// I - File pointer
+	     const char *base)		// I - Base directory for file
 {
-  int		ch;		/* Character from file */
-  uchar		*ptr,		/* Pointer in string */
-		glyph[16],	/* Glyph name (&#nnn; or &name;) */
-		*glyphptr;	/* Pointer in glyph string */
-  tree_t	*tree,		/* "top" of this tree */
-		*t,		/* New tree entry */
-		*prev,		/* Previous tree entry */
-		*temp;		/* Temporary looping var */
-  int		descend;	/* Descend into node? */
-  FILE		*embed;		/* File pointer for EMBED */
-  char		newbase[1024];	/* New base directory for EMBED */
-  uchar		*filename,	/* Filename for EMBED tag */
-		*face,		/* Typeface for FONT tag */
-		*color,		/* Color for FONT tag */
-		*size,		/* Size for FONT tag */
-		*type;		/* Type for EMBED tag */
-  int		sizeval;	/* Size value from FONT tag */
-  int		linenum;	/* Line number in file */
-  static uchar	s[10240];	/* String from file */
-  static int	have_whitespace = 0;
-  				/* Non-zero if there was leading whitespace */
+  int		ch;			// Character from file
+  uchar		*ptr,			// Pointer in string
+		entity[16],		// Character entity name (&#nnn; or &name;)
+		*eptr;			// Pointer in entity string
+  tree_t	*tree,			// "top" of this tree
+		*t,			// New tree entry
+		*prev,			// Previous tree entry
+		*temp;			// Temporary looping var
+  int		descend;		// Descend into node?
+  FILE		*embed;			// File pointer for EMBED
+  char		newbase[1024];		// New base directory for EMBED
+  uchar		*filename,		// Filename for EMBED tag
+		*face,			// Typeface for FONT tag
+		*color,			// Color for FONT tag
+		*size,			// Size for FONT tag
+		*type;			// Type for EMBED tag
+  int		sizeval;		// Size value from FONT tag
+  int		linenum;		// Line number in file
+  static uchar	s[10240];		// String from file
+  static int	have_whitespace = 0;	// Non-zero if there was leading whitespace
 
 
   DEBUG_printf(("htmlReadFile(parent=%p, fp=%p, base=\"%s\")\n",
@@ -348,7 +347,7 @@ htmlReadFile(tree_t     *parent,/* I - Parent tree entry */
     if (parent == NULL)
     {
       t->halignment   = ALIGN_LEFT;
-      t->valignment   = ALIGN_MIDDLE;
+      t->valignment   = ALIGN_BOTTOM;
       t->typeface     = _htmlBodyFont;
       t->size         = SIZE_P;
 
@@ -637,7 +636,7 @@ htmlReadFile(tree_t     *parent,/* I - Parent tree entry */
 	    if (parent == NULL)
 	    {
 	      t->halignment   = ALIGN_LEFT;
-	      t->valignment   = ALIGN_MIDDLE;
+	      t->valignment   = ALIGN_BOTTOM;
 	      t->typeface     = _htmlBodyFont;
 	      t->size         = SIZE_P;
 
@@ -693,39 +692,56 @@ htmlReadFile(tree_t     *parent,/* I - Parent tree entry */
       {
         if (ch == '&')
         {
-          for (glyphptr = glyph;
-               (ch = getc(fp)) != EOF && (glyphptr - glyph) < 15;
-               glyphptr ++)
-            if (ch == ';' || isspace(ch))
-	    {
-	      if (ch == '\n')
-	        linenum ++;
+	  // Possibly a character entity...
+	  eptr = entity;
+	  while (eptr < (entity + sizeof(entity) - 1) &&
+	         (ch = getc(fp)) != EOF)
+	    if (!isalnum(ch) && ch != '#')
+	      break;
+	    else
+	      *eptr++ = ch;
 
-              break;
-	    }
-            else
-              *glyphptr = ch;
+          if (ch != ';')
+	  {
+	    ungetc(ch, fp);
+	    ch = 0;
+	  }
 
-          if (glyphptr == glyph)
+          *eptr = '\0';
+          if (!ch)
 	  {
 	    progress_error(HD_ERROR_HTML_ERROR, "Unquoted & on line %d.",
 	                   linenum);
-            ch = '&';
-	  }
-	  else
-	  {
-            *glyphptr = '\0';
-            ch = iso8859(glyph);
-	  }
-        }
 
-        if (ch != 0 && ch != '\r')
+            if (ptr < (s + sizeof(s) - 1))
+	      *ptr++ = '&';
+            strlcpy((char *)ptr, (char *)entity, sizeof(s) - (ptr - s));
+	    ptr += strlen((char *)ptr);
+	  }
+	  else if ((ch = iso8859(entity)) == 0)
+	  {
+	    progress_error(HD_ERROR_HTML_ERROR, "Unknown character entity \"&%s;\" on line %d.",
+	                   entity, linenum);
+
+            if (ptr < (s + sizeof(s) - 1))
+	      *ptr++ = '&';
+            strlcpy((char *)ptr, (char *)entity, sizeof(s) - (ptr - s));
+	    ptr += strlen((char *)ptr);
+            if (ptr < (s + sizeof(s) - 1))
+	      *ptr++ = ';';
+	  }
+	  else if (ptr < (s + sizeof(s) - 1))
+	    *ptr++ = ch;
+        }
+	else if (ch != 0 && ch != '\r')
+	{
           *ptr++ = ch;
 
-        if (ch == '\n')
-	{
-	  linenum ++;
-          break;
+          if (ch == '\n')
+	  {
+	    linenum ++;
+            break;
+	  }
 	}
 
         ch = getc(fp);
@@ -758,28 +774,48 @@ htmlReadFile(tree_t     *parent,/* I - Parent tree entry */
       {
         if (ch == '&')
         {
-          for (glyphptr = glyph;
-               (ch = getc(fp)) != EOF && (glyphptr - glyph) < 15;
-               glyphptr ++)
-            if (ch == ';' || isspace(ch))
-              break;
-            else
-              *glyphptr = ch;
+	  // Possibly a character entity...
+	  eptr = entity;
+	  while (eptr < (entity + sizeof(entity) - 1) &&
+	         (ch = getc(fp)) != EOF)
+	    if (!isalnum(ch) && ch != '#')
+	      break;
+	    else
+	      *eptr++ = ch;
 
-          if (glyphptr == glyph)
+          if (ch != ';')
+	  {
+	    ungetc(ch, fp);
+	    ch = 0;
+	  }
+
+          *eptr = '\0';
+          if (!ch)
 	  {
 	    progress_error(HD_ERROR_HTML_ERROR, "Unquoted & on line %d.",
 	                   linenum);
-            ch = '&';
-	  }
-	  else
-	  {
-            *glyphptr = '\0';
-            ch = iso8859(glyph);
-	  }
-        }
 
-        if (ch != 0)
+            if (ptr < (s + sizeof(s) - 1))
+	      *ptr++ = '&';
+            strlcpy((char *)ptr, (char *)entity, sizeof(s) - (ptr - s));
+	    ptr += strlen((char *)ptr);
+	  }
+	  else if ((ch = iso8859(entity)) == 0)
+	  {
+	    progress_error(HD_ERROR_HTML_ERROR, "Unknown character entity \"&%s;\" on line %d.",
+	                   entity, linenum);
+
+            if (ptr < (s + sizeof(s) - 1))
+	      *ptr++ = '&';
+            strlcpy((char *)ptr, (char *)entity, sizeof(s) - (ptr - s));
+	    ptr += strlen((char *)ptr);
+            if (ptr < (s + sizeof(s) - 1))
+	      *ptr++ = ';';
+	  }
+	  else if (ptr < (s + sizeof(s) - 1))
+	    *ptr++ = ch;
+        }
+	else if (ch)
           *ptr++ = ch;
 
         ch = getc(fp);
@@ -847,7 +883,7 @@ htmlReadFile(tree_t     *parent,/* I - Parent tree entry */
 
           if ((color = htmlGetVariable(t, (uchar *)"BGCOLOR")) != NULL &&
 	      !BodyColor[0])
-	    strcpy(BodyColor, (char *)color);
+	    strlcpy(BodyColor, (char *)color, sizeof(BodyColor));
 
           // Update the background image as necessary...
           if ((filename = htmlGetVariable(t, (uchar *)"BACKGROUND")) != NULL)
@@ -868,7 +904,7 @@ htmlReadFile(tree_t     *parent,/* I - Parent tree entry */
 	  }
 
           // Get the image alignment...
-          t->valignment = ALIGN_TOP;
+          t->valignment = ALIGN_BOTTOM;
           get_alignment(t);
 
           // Update the image source as necessary...
@@ -995,7 +1031,7 @@ htmlReadFile(tree_t     *parent,/* I - Parent tree entry */
 
             if ((embed = fopen((char *)filename, "r")) != NULL)
             {
-	      strcpy(newbase, file_directory((char *)filename));
+	      strlcpy(newbase, file_directory((char *)filename), sizeof(newbase));
 
               htmlReadFile(t, embed, newbase);
               fclose(embed);
@@ -1015,6 +1051,11 @@ htmlReadFile(tree_t     *parent,/* I - Parent tree entry */
 	  else
             t->halignment = ALIGN_CENTER;
 
+          if (htmlGetVariable(t->parent, (uchar *)"VALIGN") != NULL)
+	    t->valignment = t->parent->valignment;
+	  else
+            t->valignment = ALIGN_MIDDLE;
+
           get_alignment(t);
 
           t->style = STYLE_BOLD;
@@ -1028,6 +1069,11 @@ htmlReadFile(tree_t     *parent,/* I - Parent tree entry */
 	  else
             t->halignment = ALIGN_LEFT;
 
+          if (htmlGetVariable(t->parent, (uchar *)"VALIGN") != NULL)
+	    t->valignment = t->parent->valignment;
+	  else
+            t->valignment = ALIGN_MIDDLE;
+
 	  get_alignment(t);
 
           t->style = STYLE_NORMAL;
@@ -1036,6 +1082,14 @@ htmlReadFile(tree_t     *parent,/* I - Parent tree entry */
           break;
 
       case MARKUP_FONT :
+          if (have_whitespace)
+	  {
+	    // Insert a space before this element...
+	    insert_space(parent, t);
+
+	    have_whitespace = 0;
+	  }
+
           if ((face = htmlGetVariable(t, (uchar *)"FACE")) != NULL)
           {
             for (ptr = face; *ptr != '\0'; ptr ++)
@@ -1047,17 +1101,7 @@ htmlReadFile(tree_t     *parent,/* I - Parent tree entry */
             else if (strstr((char *)face, "times") != NULL)
               t->typeface = TYPE_TIMES;
             else if (strstr((char *)face, "courier") != NULL)
-	    {
               t->typeface = TYPE_COURIER;
-
-              if (have_whitespace)
-	      {
-		// Insert a space before monospaced text...
-		insert_space(parent, t);
-
-		have_whitespace = 0;
-	      }
-            }
 	    else if (strstr((char *)face, "symbol") != NULL)
               t->typeface = TYPE_SYMBOL;
           }
@@ -1210,7 +1254,6 @@ htmlReadFile(tree_t     *parent,/* I - Parent tree entry */
       case MARKUP_STRONG :
           t->style = (style_t)(t->style | STYLE_BOLD);
       case MARKUP_CITE :
-      case MARKUP_DT :
       case MARKUP_EM :
       case MARKUP_I :
           t->style = (style_t)(t->style | STYLE_ITALIC);
@@ -1255,6 +1298,18 @@ htmlReadFile(tree_t     *parent,/* I - Parent tree entry */
           descend = 1;
           break;
 
+      case MARKUP_A :
+          if (have_whitespace)
+	  {
+	    // Insert a space before this link...
+	    insert_space(parent, t);
+
+	    have_whitespace = 0;
+	  }
+
+          descend = 1;
+	  break;
+
       default :
          /*
           * All other markup types should be using <MARK>...</MARK>
@@ -1269,7 +1324,7 @@ htmlReadFile(tree_t     *parent,/* I - Parent tree entry */
     if (descend)
     {
 #ifdef DEBUG
-      strcat((char *)indent, "    ");
+      strlcat((char *)indent, "    ", sizeof(indent));
 #endif // DEBUG
 
       parent = t;
@@ -1607,7 +1662,7 @@ htmlNewTree(tree_t   *parent,	/* I - Parent entry */
   if (parent == NULL)
   {
     t->halignment = ALIGN_LEFT;
-    t->valignment = ALIGN_MIDDLE;
+    t->valignment = ALIGN_BOTTOM;
     t->typeface   = _htmlBodyFont;
     t->size       = SIZE_P;
 
@@ -2033,7 +2088,7 @@ htmlSetCharSet(const char *cs)	/* I - Character set file to load */
   int		chars[256];	/* Character encoding array */
 
 
-  strcpy(_htmlCharSet, cs);
+  strlcpy(_htmlCharSet, cs, sizeof(_htmlCharSet));
 
   if (!_htmlInitialized)
   {
@@ -2205,8 +2260,7 @@ htmlSetCharSet(const char *cs)	/* I - Character set file to load */
 void
 htmlSetTextColor(uchar *color)	/* I - Text color */
 {
-  strncpy((char *)_htmlTextColor, (char *)color, sizeof(_htmlTextColor));
-  _htmlTextColor[sizeof(_htmlTextColor) - 1] = '\0';
+  strlcpy((char *)_htmlTextColor, (char *)color, sizeof(_htmlTextColor));
 }
 
 
@@ -2347,6 +2401,16 @@ parse_markup(tree_t *t,		/* I - Current tree entry */
   while ((ch = getc(fp)) != EOF && mptr < (markup + sizeof(markup) - 1))
     if (ch == '>' || isspace(ch))
       break;
+    else if (ch == '/' && mptr > markup)
+    {
+      // Look for "/>"...
+      ch = getc(fp);
+
+      if (ch != '>')
+        return (MARKUP_ERROR);
+
+      break;
+    }
     else
     {
       *mptr++ = ch;
@@ -2377,7 +2441,7 @@ parse_markup(tree_t *t,		/* I - Current tree entry */
     */
 
     t->markup = MARKUP_UNKNOWN;
-    strcpy((char *)comment, (char *)markup);
+    strlcpy((char *)comment, (char *)markup, sizeof(comment));
     cptr = comment + strlen((char *)comment);
 
     DEBUG_printf(("%s%s (unrecognized!)\n", indent, markup));
@@ -2397,13 +2461,13 @@ parse_markup(tree_t *t,		/* I - Current tree entry */
       if (ch == '>' && temp == NULL)
         break;
 
-      *cptr++ = ch;
-
       if (ch == '\n')
         (*linenum) ++;
 
       if (ch == '-')
       {
+        *cptr++ = ch;
+
         if ((ch2 = getc(fp)) == '>')
 	{
 	  // Erase trailing -->
@@ -2417,7 +2481,61 @@ parse_markup(tree_t *t,		/* I - Current tree entry */
 	  ch = ch2;
       }
       else
+      {
+        if (ch == '&')
+	{
+          // Handle character entities...
+	  uchar	entity[16],		// Character entity name
+		*eptr;			// Pointer into name
+
+
+	  eptr = entity;
+	  while (eptr < (entity + sizeof(entity) - 1) &&
+		 (ch = getc(fp)) != EOF)
+	    if (!isalnum(ch) && ch != '#')
+	      break;
+	    else
+	      *eptr++ = ch;
+
+	  if (ch != ';')
+	  {
+	    ungetc(ch, fp);
+	    ch = 0;
+	  }
+
+	  *eptr = '\0';
+	  if (!ch)
+	  {
+	    progress_error(HD_ERROR_HTML_ERROR, "Unquoted & on line %d.",
+	                   *linenum);
+
+            if (cptr < (comment + sizeof(comment) - 1))
+	      *cptr++ = '&';
+            strlcpy((char *)cptr, (char *)entity,
+	            sizeof(comment) - (cptr - comment));
+	    cptr += strlen((char *)cptr);
+	  }
+	  else if ((ch = iso8859(entity)) == 0)
+	  {
+	    progress_error(HD_ERROR_HTML_ERROR, "Unknown character entity \"&%s;\" on line %d.",
+	                   entity, *linenum);
+
+            if (cptr < (comment + sizeof(comment) - 1))
+	      *cptr++ = '&';
+            strlcpy((char *)cptr, (char *)entity,
+	            sizeof(comment) - (cptr - comment));
+	    cptr += strlen((char *)cptr);
+            if (cptr < (comment + sizeof(comment) - 1))
+	      *cptr++ = ';';
+	  }
+	  else if (cptr < (comment + sizeof(comment) - 1))
+	    *cptr++ = ch;
+	}
+	else
+	  *cptr++ = ch;
+
         ch = getc(fp);
+      }
     }
 
     *cptr = '\0';
@@ -2437,6 +2555,17 @@ parse_markup(tree_t *t,		/* I - Current tree entry */
       }
 
       ch = getc(fp);
+
+      if (ch == '/')
+      {
+	// Look for "/>"...
+	ch = getc(fp);
+
+	if (ch != '>')
+          return (MARKUP_ERROR);
+
+	break;
+      }
     }
   }
 
@@ -2448,21 +2577,25 @@ parse_markup(tree_t *t,		/* I - Current tree entry */
  * 'parse_variable()' - Parse a markup variable string.
  */
 
-static int			/* O - -1 on error, 0 on success */
-parse_variable(tree_t *t,	/* I - Current tree entry */
-               FILE   *fp,	/* I - Input file */
-	       int    *linenum)	/* I - Current line number */
+static int				// O - -1 on error, 0 on success
+parse_variable(tree_t *t,		// I - Current tree entry
+               FILE   *fp,		// I - Input file
+	       int    *linenum)		// I - Current line number
 {
-  uchar	name[1024],		/* Name of variable */
-	value[10240],		/* Value of variable */
-	*ptr;			/* Temporary pointer */
-  int	ch;			/* Character from file */
+  uchar	name[1024],			// Name of variable
+	value[10240],			// Value of variable
+	*ptr,				// Temporary pointer
+	entity[16],			// Character entity name
+	*eptr;				// Pointer into name
+  int	ch;				// Character from file
 
 
   ptr = name;
   while ((ch = getc(fp)) != EOF)
     if (isspace(ch) || ch == '=' || ch == '>' || ch == '\r')
       break;
+    else if (ch == '/' && ptr == name)
+      break;
     else if (ptr < (name + sizeof(name) - 1))
       *ptr++ = ch;
 
@@ -2499,26 +2632,126 @@ parse_variable(tree_t *t,	/* I - Current tree entry */
         if (ch == '\'')
         {
           while ((ch = getc(fp)) != EOF)
+	  {
             if (ch == '\'')
               break;
+	    else if (ch == '&')
+	    {
+	      // Possibly a character entity...
+	      eptr = entity;
+	      while (eptr < (entity + sizeof(entity) - 1) &&
+	             (ch = getc(fp)) != EOF)
+	        if (!isalnum(ch) && ch != '#')
+		  break;
+		else
+		  *eptr++ = ch;
+
+              if (ch != ';')
+	      {
+	        ungetc(ch, fp);
+		ch = 0;
+	      }
+
+              *eptr = '\0';
+              if (!ch)
+	      {
+		progress_error(HD_ERROR_HTML_ERROR, "Unquoted & on line %d.",
+	                       *linenum);
+
+        	if (ptr < (value + sizeof(value) - 1))
+		  *ptr++ = '&';
+                strlcpy((char *)ptr, (char *)entity, sizeof(value) - (ptr - value));
+		ptr += strlen((char *)ptr);
+	      }
+	      else if ((ch = iso8859(entity)) == 0)
+	      {
+		progress_error(HD_ERROR_HTML_ERROR, "Unknown character entity \"&%s;\" on line %d.",
+	                       entity, *linenum);
+
+        	if (ptr < (value + sizeof(value) - 1))
+		  *ptr++ = '&';
+                strlcpy((char *)ptr, (char *)entity, sizeof(value) - (ptr - value));
+		ptr += strlen((char *)ptr);
+        	if (ptr < (value + sizeof(value) - 1))
+		  *ptr++ = ';';
+	      }
+	      else if (ptr < (value + sizeof(value) - 1))
+	        *ptr++ = ch;
+	    }
             else if (ptr < (value + sizeof(value) - 1) &&
 	             ch != '\n' && ch != '\r')
               *ptr++ = ch;
 	    else if (ch == '\n')
+	    {
+	      if (ptr < (value + sizeof(value) - 1))
+	        *ptr++ = ' ';
+
 	      (*linenum) ++;
+	    }
+	  }
 
           *ptr = '\0';
         }
         else if (ch == '\"')
         {
           while ((ch = getc(fp)) != EOF)
+	  {
             if (ch == '\"')
               break;
+	    else if (ch == '&')
+	    {
+	      // Possibly a character entity...
+	      eptr = entity;
+	      while (eptr < (entity + sizeof(entity) - 1) &&
+	             (ch = getc(fp)) != EOF)
+	        if (!isalnum(ch) && ch != '#')
+		  break;
+		else
+		  *eptr++ = ch;
+
+              if (ch != ';')
+	      {
+	        ungetc(ch, fp);
+		ch = 0;
+	      }
+
+              *eptr = '\0';
+              if (!ch)
+	      {
+		progress_error(HD_ERROR_HTML_ERROR, "Unquoted & on line %d.",
+	                       *linenum);
+
+        	if (ptr < (value + sizeof(value) - 1))
+		  *ptr++ = '&';
+                strlcpy((char *)ptr, (char *)entity, sizeof(value) - (ptr - value));
+		ptr += strlen((char *)ptr);
+	      }
+	      else if ((ch = iso8859(entity)) == 0)
+	      {
+		progress_error(HD_ERROR_HTML_ERROR, "Unknown character entity \"&%s;\" on line %d.",
+	                       entity, *linenum);
+
+        	if (ptr < (value + sizeof(value) - 1))
+		  *ptr++ = '&';
+                strlcpy((char *)ptr, (char *)entity, sizeof(value) - (ptr - value));
+		ptr += strlen((char *)ptr);
+        	if (ptr < (value + sizeof(value) - 1))
+		  *ptr++ = ';';
+	      }
+	      else if (ptr < (value + sizeof(value) - 1))
+	        *ptr++ = ch;
+	    }
             else if (ptr < (value + sizeof(value) - 1) &&
 	             ch != '\n' && ch != '\r')
               *ptr++ = ch;
 	    else if (ch == '\n')
+	    {
+	      if (ptr < (value + sizeof(value) - 1))
+	        *ptr++ = ' ';
+
 	      (*linenum) ++;
+	    }
+	  }
 
           *ptr = '\0';
         }
@@ -2526,10 +2759,55 @@ parse_variable(tree_t *t,	/* I - Current tree entry */
         {
           *ptr++ = ch;
           while ((ch = getc(fp)) != EOF)
+	  {
             if (isspace(ch) || ch == '>' || ch == '\r')
               break;
+	    else if (ch == '&')
+	    {
+	      // Possibly a character entity...
+	      eptr = entity;
+	      while (eptr < (entity + sizeof(entity) - 1) &&
+	             (ch = getc(fp)) != EOF)
+	        if (!isalnum(ch) && ch != '#')
+		  break;
+		else
+		  *eptr++ = ch;
+
+              if (ch != ';')
+	      {
+	        ungetc(ch, fp);
+		ch = 0;
+	      }
+
+              *eptr = '\0';
+              if (!ch)
+	      {
+		progress_error(HD_ERROR_HTML_ERROR, "Unquoted & on line %d.",
+	                       *linenum);
+
+        	if (ptr < (value + sizeof(value) - 1))
+		  *ptr++ = '&';
+                strlcpy((char *)ptr, (char *)entity, sizeof(value) - (ptr - value));
+		ptr += strlen((char *)ptr);
+	      }
+	      else if ((ch = iso8859(entity)) == 0)
+	      {
+		progress_error(HD_ERROR_HTML_ERROR, "Unknown character entity \"&%s;\" on line %d.",
+	                       entity, *linenum);
+
+        	if (ptr < (value + sizeof(value) - 1))
+		  *ptr++ = '&';
+                strlcpy((char *)ptr, (char *)entity, sizeof(value) - (ptr - value));
+		ptr += strlen((char *)ptr);
+        	if (ptr < (value + sizeof(value) - 1))
+		  *ptr++ = ';';
+	      }
+	      else if (ptr < (value + sizeof(value) - 1))
+	        *ptr++ = ch;
+	    }
             else if (ptr < (value + sizeof(value) - 1))
               *ptr++ = ch;
+	  }
 
 	  if (ch == '\n')
 	    (*linenum) ++;
@@ -2592,7 +2870,7 @@ compute_size(tree_t *t)		/* I - Tree entry */
       sprintf(number, "%d",
               atoi((char *)width_ptr) * img->height / img->width);
       if (strchr((char *)width_ptr, '%') != NULL)
-        strcat(number, "%");
+        strlcat(number, "%", sizeof(number));
       htmlSetVariable(t, (uchar *)"HEIGHT", (uchar *)number);
     }
     else if (height_ptr != NULL)
@@ -2603,7 +2881,7 @@ compute_size(tree_t *t)		/* I - Tree entry */
       sprintf(number, "%d",
               atoi((char *)height_ptr) * img->width / img->height);
       if (strchr((char *)height_ptr, '%') != NULL)
-        strcat(number, "%");
+        strlcat(number, "%", sizeof(number));
       htmlSetVariable(t, (uchar *)"WIDTH", (uchar *)number);
     }
     else
@@ -2780,41 +3058,26 @@ fix_filename(char *filename,		/* I - Original filename */
   if (strcmp(base, ".") == 0 || strstr(filename, "//") != NULL)
     return (file_find(Path, filename));
 
-#ifdef MAC
-  //
-  // Convert UNIX/DOS/WINDOWS slashes to colons for MacOS...
-  //
-  // Question: WHY doesn't the Mac standard C library do this for
-  // you???
-  //
-
-  for (slash = strchr(filename, '/'); slash != NULL; slash = strchr(slash + 1, '/'))
-    *slash = ':';
-
-  for (slash = strchr(filename, '\\'); slash != NULL; slash = strchr(slash + 1, '\\'))
-    *slash = ':';
-#endif // MAC
-
   if (strncmp(filename, "./", 2) == 0 ||
       strncmp(filename, ".\\", 2) == 0)
     filename += 2;
 
   if (strncmp(base, "http://", 7) == 0 || strncmp(base, "https://", 8) == 0)
   {
-    strcpy(newfilename, base);
+    strlcpy(newfilename, base, sizeof(newfilename));
     base = strchr(newfilename, ':') + 3;
 
     if (filename[0] == '/')
     {
       if ((slash = strchr(base, '/')) != NULL)
-        strcpy(slash, filename);
+        strlcpy(slash, filename, sizeof(newfilename) - (slash - newfilename));
       else
-        strcat(newfilename, filename);
+        strlcat(newfilename, filename, sizeof(newfilename));
 
       return (newfilename);
     }
     else if ((slash = strchr(base, '/')) == NULL)
-      strcat(newfilename, "/");
+      strlcat(newfilename, "/", sizeof(newfilename));
   }
   else
   {
@@ -2822,30 +3085,13 @@ fix_filename(char *filename,		/* I - Original filename */
 	base[0] == '\0' || (isalpha(filename[0]) && filename[1] == ':'))
       return (file_find(Path, filename)); /* No change needed for absolute path */
 
-    strcpy(newfilename, base);
+    strlcpy(newfilename, base, sizeof(newfilename));
     base = newfilename;
   }
 
-#ifdef MAC
-  //
-  // Convert UNIX/DOS/WINDOWS slashes to colons for MacOS...
-  //
-  // Question: WHY doesn't the Mac standard C library do this for
-  // you???
-  //
-
-  for (slash = strchr(newfilename, '/'); slash != NULL; slash = strchr(slash + 1, '/'))
-    *slash = ':';
-
-  for (slash = strchr(newfilename, '\\'); slash != NULL; slash = strchr(slash + 1, '\\'))
-    *slash = ':';
-#endif // MAC
-
 #if defined(WIN32) || defined(__EMX__)
   while (strncmp(filename, "../", 3) == 0 ||
          strncmp(filename, "..\\", 3) == 0)
-#elif defined(MAC)
-  while (strncmp(filename, "..:", 3) == 0)
 #else
   while (strncmp(filename, "../", 3) == 0)
 #endif // WIN32 || __EMX__
@@ -2856,9 +3102,6 @@ fix_filename(char *filename,		/* I - Original filename */
       *slash = '\0';
     else if ((slash = strrchr(base, '\\')) != NULL)
       *slash = '\0';
-#elif defined(MAC)
-    if ((slash = strrchr(base, ':')) != NULL)
-      *slash = '\0';
 #else
     if ((slash = strrchr(base, '/')) != NULL)
       *slash = '\0';
@@ -2870,14 +3113,10 @@ fix_filename(char *filename,		/* I - Original filename */
     }
   }
 
-#ifdef MAC
-  strcat(newfilename, ":");
-#else
   if (filename[0] != '/' && *base && base[strlen(base) - 1] != '/')
-    strcat(newfilename, "/");
-#endif // MAC
+    strlcat(newfilename, "/", sizeof(newfilename));
 
-  strcat(newfilename, filename);
+  strlcat(newfilename, filename, sizeof(newfilename));
 
   return (file_find(Path, newfilename));
 }
@@ -2944,6 +3183,119 @@ htmlDebugStats(const char *title,	// I - Title
 }
 
 
+//
+// 'htmlFindFile()' - Find a file in the document.
+//
+
+tree_t *				// O - Node for file
+htmlFindFile(tree_t *doc,		// I - Document pointer
+             uchar  *filename)		// I - Filename
+{
+  tree_t	*tree;			// Current node
+  uchar		*treename;		// Filename from node
+
+
+  if (!filename || !doc)
+    return (NULL);
+
+  for (tree = doc; tree; tree = tree->next)
+    if ((treename = htmlGetVariable(tree, (uchar *)"_HD_FILENAME")) != NULL &&
+        !strcmp((char *)treename, (char *)filename))
+      return (tree);
+
+  return (NULL);
+}
+
+
+//
+// 'htmlFixLinks()' - Fix the external links in the document.
+//
+
+void
+htmlFixLinks(tree_t *doc,		// I - Top node
+             tree_t *tree,		// I - Current node
+	     uchar  *base)		// I - Base directory/path
+{
+  uchar		*href;			// HREF attribute
+  char		full_href[1024];	// Full HREF value
+  const char	*debug;			// HTMLDOC_DEBUG environment variable
+  static int	show_debug = -1;	// Show debug messages?
+
+
+  if (show_debug < 0)
+  {
+    if ((debug = getenv("HTMLDOC_DEBUG")) == NULL ||
+	(strstr(debug, "all") == NULL && strstr(debug, "links") == NULL))
+      show_debug = 0;
+    else
+      show_debug = 1;
+  }
+
+  while (tree)
+  {
+    if (tree->markup == MARKUP_A && base && base[0] &&
+        (href = htmlGetVariable(tree, (uchar *)"HREF")) != NULL)
+    {
+      // Check if the link needs to be localized...
+      if (href[0] != '#' && file_method((char *)href) == NULL &&
+          file_method((char *)base) != NULL &&
+	  htmlFindFile(doc, (uchar *)file_basename((char *)href)) == NULL)
+      {
+        // Yes, localize it...
+	if (href[0] == '/')
+	{
+	  // Absolute URL, just copy scheme, server, etc.
+	  char *ptr;			// Pointer into URL...
+
+	  strlcpy(full_href, (char *)base, sizeof(full_href));
+
+          if (href[1] == '/')
+	  {
+	    // Just use scheme...
+	    if ((ptr = strstr(full_href, "//")) != NULL)
+	      *ptr ='\0';
+	  }
+	  else if ((ptr = strstr(full_href, "//")) != NULL  &&
+	           (ptr = strchr(ptr + 2, '/')) != NULL)
+	    *ptr ='\0';
+
+	  strlcat(full_href, (char *)href, sizeof(full_href));
+	}
+	else if (!strncmp((char *)href, "./", 2))
+	{
+	  // Relative URL of the form "./foo/bar", append href sans
+	  // "./" to base to form full href...
+	  snprintf(full_href, sizeof(full_href), "%s/%s", base, href + 2);
+	}
+	else
+	{
+	  // Relative URL, append href to base to form full href...
+	  snprintf(full_href, sizeof(full_href), "%s/%s", base, href);
+	}
+
+        if (show_debug)
+          progress_error(HD_ERROR_NONE, "DEBUG: Mapping \"%s\" to \"%s\"...\n",
+	        	 href, full_href);
+
+	htmlSetVariable(tree, (uchar *)"_HD_FULL_HREF", (uchar *)full_href);
+      }
+      else
+      {
+        // No, just mirror the link in the _HD_FULL_HREF attribute...
+	htmlSetVariable(tree, (uchar *)"_HD_FULL_HREF", href);
+      }
+    }
+    else if (tree->markup == MARKUP_FILE)
+      base = htmlGetVariable(tree, (uchar *)"_HD_BASE");
+
+    if (tree->child)
+      htmlFixLinks(doc, tree->child, base);
+
+    tree = tree->next;
+  }
+}
+
+
 /*
- * End of "$Id: htmllib.cxx,v 1.41.2.64 2002/10/11 14:23:28 mike Exp $".
+ * End of "$Id: htmllib.cxx,v 1.41.2.79 2004/10/21 19:05:04 mike Exp $".
  */
