@@ -1,10 +1,10 @@
 /*
- * "$Id: iso8859.cxx,v 1.3.2.7 2002/07/29 16:48:23 mike Exp $"
+ * "$Id: iso8859.cxx,v 1.3.2.11 2004/05/09 15:04:38 mike Exp $"
  *
  *   ISO-8859-1 conversion routines for HTMLDOC, an HTML document
  *   processing program.
  *
- *   Copyright 1997-2002 by Easy Software Products.
+ *   Copyright 1997-2004 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -16,7 +16,7 @@
  *       Attn: ESP Licensing Information
  *       Easy Software Products
  *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636-3111 USA
+ *       Hollywood, Maryland 20636-3142 USA
  *
  *       Voice: (301) 373-9600
  *       EMail: info@easysw.com
@@ -195,7 +195,7 @@ iso8859(uchar *name)		/* I - Glyph name */
     return (ch);
   }
 
-  strcpy((char *)key.name, (char *)name);
+  strlcpy((char *)key.name, (char *)name, sizeof(key.name));
   match = (lut_t *)bsearch(&key, iso8859_numbers,
                            sizeof(iso8859_numbers) / sizeof(iso8859_numbers[0]),
                            sizeof(iso8859_numbers[0]),
@@ -254,5 +254,5 @@ compare_lut(lut_t *a,	/* I - First glyph */
 
 
 /*
- * End of "$Id: iso8859.cxx,v 1.3.2.7 2002/07/29 16:48:23 mike Exp $".
+ * End of "$Id: iso8859.cxx,v 1.3.2.11 2004/05/09 15:04:38 mike Exp $".
  */
