@@ -1,9 +1,9 @@
 /*
- * "$Id: progress.cxx,v 1.6.2.12 2002/07/31 19:11:11 mike Exp $"
+ * "$Id: progress.cxx,v 1.6.2.17 2004/05/25 18:40:35 mike Exp $"
  *
  *   Progress functions for HTMLDOC, a HTML document processing program.
  *
- *   Copyright 1997-2002 by Easy Software Products.
+ *   Copyright 1997-2004 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -15,7 +15,7 @@
  *       Attn: ESP Licensing Information
  *       Easy Software Products
  *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636-3111 USA
+ *       Hollywood, Maryland 20636-3142 USA
  *
  *       Voice: (301) 373-9600
  *       EMail: info@easysw.com
@@ -40,6 +40,12 @@
 #  include <FL/fl_ask.H>
 #endif // HAVE_LIBFLTK
 
+#ifdef WIN32
+#  define getpid	GetCurrentProcessId
+#else
+#  include <unistd.h>
+#endif // WIN32
+
 
 /*
  * Local globals...
@@ -81,15 +87,27 @@ progress_error(HDerror    error,	/* I - Error number */
   }
 #endif /* HAVE_LIBFLTK */
 
+#ifdef WIN32
+  // IIS doesn't separate stderr from stdout, so we cannot output any CGI error messages
+  // on Windows...
+  if (CGIMode)
+    return;
+#endif // WIN32
+
   if (Verbosity >= 0)
   {
     if (progress_visible)
       fprintf(stderr, "\r%-79.79s\r", "");
 
+    if (CGIMode)
+      fprintf(stderr, "HTMLDOC(%d) ", getpid());
+
     if (error)
       fprintf(stderr, "ERR%03d: %s\n", error, text);
     else
       fprintf(stderr, "%s\n", text);
+
+    fflush(stderr);
   }
 }
 
@@ -109,6 +127,9 @@ progress_hide(void)
   }
 #endif /* HAVE_LIBFLTK */
 
+  if (CGIMode)
+    return;
+
   if (Verbosity > 0)
   {
     fprintf(stderr, "\r%-79.79s\r", "");
@@ -143,6 +164,17 @@ progress_show(const char *format,	/* I - Printf-style format string */
   }
 #endif /* HAVE_LIBFLTK */
 
+  if (CGIMode)
+  {
+    if (Verbosity > 0)
+    {
+      fprintf(stderr, "HTMLDOC(%d) INFO: %s\n", getpid(), text);
+      fflush(stderr);
+    }
+
+    return;
+  }
+
   if (Verbosity > 0)
   {
     fprintf(stderr, "\r%-79.79s", text);
@@ -171,5 +203,5 @@ progress_update(int percent)	/* I - Percent complete */
 
 
 /*
- * End of "$Id: progress.cxx,v 1.6.2.12 2002/07/31 19:11:11 mike Exp $".
+ * End of "$Id: progress.cxx,v 1.6.2.17 2004/05/25 18:40:35 mike Exp $".
  */
